@@ -20,7 +20,7 @@ from inspect import signature
 from xml.etree import ElementTree as ET
 
 from absl import app, flags
-from devapp.tools import tabulate, termwidth
+from devapp.tools import tabulate, termwidth, action_flags, define_flags
 
 
 def func_from_partial(f):
@@ -135,13 +135,18 @@ def term_widths(have_match):
 
 
 def term_line(line_spec, widths, have_match):
-    """One line of output, """
+    """One line of output - have_match typically [main module name] """
     m = line_spec
+    is_action = m.get('action')
+    # if is_action:
+    #     breakpoint()  # FIXME BREAKPOINT
     r = []
     pos = 0
     for colmn in 'short_name', 'name', 'default', 'meaning':
         v = str(m.get(colmn)) or ''
-        v = "''" if v == 'None' else v
+        if v == 'None':
+            v = '' if colmn == 'short_name' else "''"
+
         if have_match:
             v += ' '
         w = widths[colmn]
@@ -159,22 +164,32 @@ def term_line(line_spec, widths, have_match):
                 det = '\n' + det
                 if have_match or len(v) + len(det) <= w:
                     v += col('details') + det
+        c = colmn
+        if is_action:
+            if m.get('action_flag'):
+                if colmn in ('short_name', 'meaning'):
+                    v = '   ' + v
+            elif colmn == 'default':
+                v, c = 'ACTION', 'action'
+                if m.get('is_default'):
+                    v += '*'
+                    c = 'default_action'
 
         if have_match:
             if colmn == 'meaning':
                 r[-1] = r[-1].rstrip()
                 v = '\n%s' % v  # (' ' * widths['short_name']) + v
-            else:
-                if 0 and len(v) > w:
-                    dedented_text = textwrap.dedent(v).strip()
-                    v = textwrap.fill(
-                        dedented_text,
-                        initial_indent=' ' * oldpos,
-                        subsequent_indent=' ' * 4,
-                        width=widths['term'],
-                    )
-                    v += ' '
-                    pos = 0
+            # else:
+            #     if 0 and len(v) > w:
+            #         dedented_text = textwrap.dedent(v).strip()
+            #         v = textwrap.fill(
+            #             dedented_text,
+            #             initial_indent=' ' * oldpos,
+            #             subsequent_indent=' ' * 4,
+            #             width=widths['term'],
+            #         )
+            #         v += ' '
+            #         pos = 0
         else:
             # if colmn == 'default' and 'mini' in v:
             #    breakpoint()  # FIXME BREAKPOINT
@@ -187,7 +202,7 @@ def term_line(line_spec, widths, have_match):
                     except Exception as ex:
                         # when not even placeholder fits
                         v = v[:w]
-        r.append(col(colmn) + v.ljust(w))
+        r.append(col(c) + v.ljust(w))
     return ''.join(r)
 
 
@@ -196,7 +211,7 @@ def to_terminal(flags, widths, match=None):
     return '\n'.join(r)
 
 
-def parse_xml_help(xml_help, match):
+def parse_xml_help(xml_help, match, cli_actions=None):
     """parse --helpxml output of absl"""
 
     class types:
@@ -223,16 +238,13 @@ def parse_xml_help(xml_help, match):
         def do_int(c, el):
             c['default'] = int(c['default'])
 
-    # open('/tmp/foo1', 'w').write(xml_help)
-    tree = ET.fromstring(xml_help).getchildren()
-    m = {}
+    tree = ET.fromstring(xml_help)
+    m, items = {}, {}
     r = {'program': tree[0].text, 'usage': tree[0].text, 'flags': m}
-    next_ = lambda els: els.pop(0)
     for fs in tree[2:]:
-        f = fs.getchildren()
+        f = fs
         c = {}
-        while f:
-            t = next_(f)
+        for t in f:
             tn = t.tag
             c[tn] = t.text
             if tn == 'type':
@@ -244,11 +256,48 @@ def parse_xml_help(xml_help, match):
                 if 'Details: ' in mn:
                     c['meaning'], c['details'] = mn.split('Details: ', 1)
                 break
-
         if match and not any([m in str(c) for m in match]):
             continue
         c.pop('current', 0)
-        m.setdefault(c['file'], []).append(c)
+        l = m.setdefault(c['file'], [])
+        l.append(c)
+        items[c['name']] = {'pos': len(l) - 1, 'mod': c['file']}
+
+    have = set()
+    for f, af in action_flags.items():
+        k = af['key']
+        if k in have:
+            continue
+        have.add(k)
+        item = items.get(k)
+        p, mod = item['pos'] + 1, item['mod']
+        m[mod][p - 1]['action'] = True
+        m[mod][p - 1]['is_default'] = af['is_default']
+        # afp.append([p - 1, mod, k, True])
+        if af in cli_actions:
+            f = m[mod][p]
+            while f['name'].startswith(k + '_'):
+                f['action'] = True
+                f['action_flag'] = True
+                f['name'] = f['name'].split(k + '_', 1)[1]
+                p += 1
+    # if afp:
+    #     fs = r['flags']
+    #     l = []
+    #     m = {}
+    #     for a in afp:
+    #         mod = a[1]
+    #         action = a[2]
+    #         main = a[3]
+    #         f = fs[mod].pop(a[0] + m.get(mod, 0))
+    #         m[mod] = m.setdefault(mod, 0) - 1
+    #         l.append(f)
+    #         if main:
+    #             f['action'] = True
+    #             f['default'] = 'ACTION'
+    #         else:
+    #             f['name'] = f['name'].split(action + '_', 1)[1]
+    #     fs['actions'] = l
     return r
 
 
@@ -278,6 +327,8 @@ col = lambda c: ansi_col(
         'choices': '0;38',
         'details': '0;38;5;241',
         'short_name': '0;32',
+        'action': '0;33',
+        'default_action': '1;33',
     }.get(c)
 )
 
@@ -300,6 +351,14 @@ def color_usage(*a, main_module, full=None, **kw):
     if dmn:
         match.append(dmn)
 
+    # -h with an action flag? Then we detail its sub flags:
+    afs = []
+    for k in sys.argv:
+        af = action_flags.get(k)
+        if af:
+            define_flags(af['flg_cls'], sub=af['key'], parent_autoshort=af['autoshort'])
+            afs.append(af)
+
     so = sys.stdout
     s = io.StringIO()
     sys.stdout = s
@@ -307,7 +366,7 @@ def color_usage(*a, main_module, full=None, **kw):
     sys.stdout = so
     u = s.getvalue()
     s.close()
-    j = parse_xml_help(u, match=match)
+    j = parse_xml_help(u, match=match, cli_actions=afs)
     all_flgs = j['flags']
     if hof != 'terminal':
         r = []
@@ -323,8 +382,8 @@ def color_usage(*a, main_module, full=None, **kw):
     j['match'] = '[matching %s]' % match_hilite if match else ''
 
     if match:
-        n = 'All supported' if full else 'Main'
-        add('%s command line flags %s:' % (n, j['match']))
+        n, n1 = ('All supported', '') if full else ('Main', ' (-hf for all flags)')
+        add('%s command line flags %s%s:' % (n, j['match'], n1))
     if hof != 'terminal':
         r = tabulate()
 
@@ -340,9 +399,11 @@ def color_usage(*a, main_module, full=None, **kw):
     # show non main module flags on helpfull:
     n = main_module.__name__
     if full:
-        [do(fn_mod) for fn_mod in sorted(all_flgs) if not fn_mod == n]
+        [do(fn_mod) for fn_mod in sorted(all_flgs) if not fn_mod in (n, 'actions')]
     if n in all_flgs:
         do(n)
+    if all_flgs.get('actions'):
+        do('actions')
 
     # show main module's flags last - always:
     add('\033[0m')
@@ -369,11 +430,15 @@ def exit_at_help_flag(main, argv):
         return
 
     mod = find_module(main)
+    if argv[-1] == mod.__name__:
+        # plugin_tools overwrite -h with -hf <module name>:
+        full = None
     if not mod:
         print('No module for function found: %s' % main)
         sys.exit(1)
+    i = argv.index(a)
     if a != '--helpfull':
-        argv[argv.index(a)] = '--helpfull'
+        argv[i] = '--helpfull'
 
     h = color_usage(main_module=mod, full=full)
     print(''.join(h))
